/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.core;

import java.io.File;
import java.io.IOException;

import org.xml.sax.SAXException;

import benten.core.io.Files;
import benten.twa.filter.core.valueobject.BentenImportHtmlProcessInput;
import benten.twa.filter.messages.BentenImportHtmlMessages;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.io.BentenTwaProcessUtil;
import benten.twa.process.BentenProcessResultInfo;
import blanco.html.normalizer.util.BlancoHtmlNormalizerUtil;
import blanco.html.parser.util.BlancoHtmlParserUtil;

/**
 * 翻訳対象のインポート
 *
 * <pre>
 * 翻訳対象物をプロジェクトにインポートします。
 *   1.  翻訳対象物のファイルをプロジェクトにインポートします。
 *   2.  インポートされた翻訳対象物は、デフォルトではプロジェクト内の [.benten/source] ディレクトリ以下に配置されます。
 *   3.  インポートされた翻訳対象物は、エクスポートの際に利用されます。
 *   4.  インポートされた翻訳対象物は、直接編集しないでください。
 * </pre>
 *
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳対象物－翻訳中間形式変換機能: HTMLインポート機能」に対応します。
 *
 * @author KASHIHARA Shinji
 * @author IGA Tosiki
 */
public class BentenImportHtmlProcessImpl extends AbstractTraverseDir implements BentenImportHtmlProcess {
	/**
	 * HTMLインポート機能のためのメッセージ。
	 */
	protected static final BentenImportHtmlMessages fMsg = new BentenImportHtmlMessages();

	/**
	 * この処理の入力オブジェクト。
	 */
	protected BentenImportHtmlProcessInput fInput;

	/**
	 * 処理の入力オブジェクトを設定。
	 * @param input 処理の入力オブジェクト。
	 */
	public void setInput(final BentenImportHtmlProcessInput input) {
		fInput = input;
	}

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	/**
	 * この処理の実行結果情報を取得します。
	 *
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	public int execute(final BentenImportHtmlProcessInput input) throws IOException, IllegalArgumentException {
		if (input == null) {
			throw new IllegalArgumentException("BentenImportHtmlProcessImpl#execute: argument 'input' is null."); //$NON-NLS-1$
		}
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		final File dirSource = new File(fInput.getSourcedir());
		if (dirSource.exists() == false) {
			throw new IllegalArgumentException(fMsg.getCoreE004(fInput.getSourcedir()));
		}
		if (dirSource.isDirectory() == false) {
			throw new IllegalArgumentException(fMsg.getCoreE005(fInput.getSourcedir()));
		}

		// トータル件数カウント。
		class FileCounter extends BentenImportHtmlProcessImpl {
			private int fCounter = 0;

			@Override
			public void processFile(final File file, final String baseDir) throws IOException {
				fCounter++;
			}

			@Override
			protected void processFilterdFile(final File file, final String baseDir) throws IOException {
				fCounter++;
			}

			/**
			 * カウンタ数の取得。
			 * @return カウンタ数。
			 */
			public int getCounter() {
				return fCounter;
			}
		}
		final FileCounter inner = new FileCounter();
		inner.setInput(input);
		inner.processDir(dirSource);
		beginTask(inner.getCounter());

		if (progress(fMsg.getCoreP002())) {
			return 6;
		}

		processDir(dirSource);

		if (progress(fMsg.getCoreP003(BentenTwaProcessUtil.getResultMessage(fResultInfo)))) {
			return 6;
		}

		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean progress(final String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		if (file.getName().toLowerCase().endsWith(".html") || file.getName().toLowerCase().endsWith(".htm")) { //$NON-NLS-1$ //$NON-NLS-2$
			return BentenTwaFilterHtmlUtil.isHtmlTagExist(file);
		} else {
			return false;
		}
	}

	@Override
	public void processFile(final File file, final String baseDir) throws IOException {
		if (fInput == null) {
			throw new IllegalArgumentException(
					"BentenImportHtmlProcessImpl#processFile: 'fInput' is null. Call execute or setInput before calling this method."); //$NON-NLS-1$
		}

		if (progress(fMsg.getCoreP011(file.getName()))) {
			return;
		}

		byte[] bytes = Files.readFileToByteArray(file);
		if (fInput.getNormalizehtml()) {
			bytes = BlancoHtmlNormalizerUtil.normalize(bytes);
		}
		final String relativePath = Files.relativePath(new File(fInput.getSourcedir()), file);
		final File outputHtmlFile = new File(fInput.getTargetdir(), relativePath);

		// 該当ディレクトリーが無い場合には、ディレクトリー作成。
		// すでに存在する場合は、それはそれで良い。
		if (outputHtmlFile.getParentFile().exists() == false) {
			if (outputHtmlFile.getParentFile().mkdirs() == false) {
				throw new IllegalArgumentException(fMsg.getCoreE006(outputHtmlFile.getParentFile().getAbsolutePath()));
			}
		} else {
			if (outputHtmlFile.getParentFile().isDirectory() == false) {
				throw new IllegalArgumentException(fMsg.getCoreE007(outputHtmlFile.getParentFile().getAbsolutePath()));
			}
		}

		// TODO HTML ノーマライズを実施しない場合には、HTML をそのまま出力するような仕様としたいです。
		// TODO このため、fInput.getNormalizehtml() が false の場合には、以下のメソッドは呼び出さないようにするのは、どうでしょうか?
		// TODO それとも、Content-type が存在しないままだとエクスポートの際に不都合が発生しますでしょうか?
		// TODO 柏原: 伊賀さんへ: HTML ノーマライズを実施しない場合、呼び出さないほうが妥当だと思います。
		// TODO 柏原: 伊賀さんへ: エクスポートは UTF-8 固定出力のため、Content-type を UTF-8 に置換する処理があります。この処理がノーマライズされていることが前提になってしまっています。
		// TODO 伊賀: HTML ノーマライズ無しの場合には　Content-type は変更しない。エンコーディング変更処理において、Content-type 無しにも対応する、という方向にしたいです。
		bytes = resolveContentType(bytes);

		Files.writeByteArrayToFile(outputHtmlFile, bytes);

		getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
	}

	@Override
	protected void processFilterdFile(final File file, final String baseDir) throws IOException {
		if (fInput == null) {
			throw new IllegalArgumentException(
					"BentenImportHtmlProcessImpl#processFilterdFile: 'fInput' is null. Call execute or setInput before calling this method."); //$NON-NLS-1$
		}

		if (progress(fMsg.getCoreP012(file.getName()))) {
			return;
		}

		final String relativePath = Files.relativePath(new File(fInput.getSourcedir()), file);
		final File outputFile = new File(fInput.getTargetdir(), relativePath);
		Files.copyFile(file, outputFile);

		getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
	}

	/**
	 * コンテンツ・タイプの解決。
	 * @param bytes HTML バイト・データ
	 * @return HTML バイト・データ
	 * @throws IOException 入出力例外が発生した場合
	 */
	private byte[] resolveContentType(byte[] bytes) throws IOException {
		try {
			final String encoding = BlancoHtmlParserUtil.decideEncoding(bytes);
			final String html = new String(bytes, encoding);

			// HTML の meta charset 指定が存在しない場合 (HTML が正規化されている前提)
			if (!html.matches("(?i)(?s)<meta\\s+http-equiv=\"Content-Type\"\\s+content=\"text/html;\\s+charset=.+?\">")) { //$NON-NLS-1$

				// meta タグの作成
				final StringBuilder metaTag = new StringBuilder();
				metaTag.append("\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset="); //$NON-NLS-1$
				metaTag.append(encoding);
				metaTag.append("\">\n"); //$NON-NLS-1$

				// head タグがない場合は作成
				String resultHtml = insertMetaTag(html, "<head>", metaTag.toString()); //$NON-NLS-1$
				if (resultHtml == null) {
					resultHtml = insertMetaTag(html, "<html>", metaTag.toString()); //$NON-NLS-1$
				}
				bytes = resultHtml.getBytes(encoding);
			}
		} catch (final SAXException e) {
			throw new IllegalStateException(e);
		}
		return bytes;
	}

	/**
	 * META タグの挿入。
	 * @param html
	 * @param beforeTag
	 * @param metaTag
	 * @return
	 */
	private String insertMetaTag(final String html, final String beforeTag, final String metaTag) {
		final StringBuilder sb = new StringBuilder();
		final int headIndex = html.toLowerCase().indexOf(beforeTag.toLowerCase());
		if (headIndex != -1) {
			sb.append(html.substring(0, headIndex + beforeTag.length()));
			sb.append(metaTag);
			sb.append(html.substring(headIndex + beforeTag.length()));
			return sb.toString();
		}
		return null;
	}
}
