/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.model;

import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 文分割リストです。
 *
 * @author KASHIHARA Shinji
 */
@SuppressWarnings("serial")
public class SentencePartitionList extends LinkedList<String> {

	/**
	 * コンストラクター。
	 * @param text テキスト
	 */
	public SentencePartitionList(final String text) {

		// 単純に分割する
		final LinkedList<String> tempList = split(text);

		// 分割したものを検査し妥当であれば、このインスタンスのリストに格納
		addAllToValidate(tempList);
	}

	/**
	 * テキストを英語の句点（「. 」「: 」）で分割します。
	 * @param text テキスト
	 * @return テキストを分割した文を要素に持つリスト
	 */
	protected LinkedList<String> split(final String text) {
		final Matcher matcher = Pattern.compile(".+?(\\.|:)\\s+").matcher(text); //$NON-NLS-1$
		final LinkedList<String> list = new LinkedList<String>();
		while (matcher.find()) {
			final StringBuffer sb = new StringBuffer();
			matcher.appendReplacement(sb, Matcher.quoteReplacement(matcher.group()));
			list.add(sb.toString());
		}
		final StringBuffer sb = new StringBuffer();
		matcher.appendTail(sb);
		list.add(sb.toString());
		return list;
	}

	/**
	 * リストの内容が文として妥当か判定し、このインスタンスのリストに追加します。
	 * 妥当でない場合は、連結されます。
	 * @param list 分割されたリスト
	 */
	protected void addAllToValidate(final LinkedList<String> list) {
		String prev = ""; //$NON-NLS-1$
		while (list.size() > 0) {
			final String s = prev + list.removeFirst();
			boolean isValid = true;

			// 囲み文字の対応を検査
			isValid = isValidSurroundChar(s, '<', '>') && isValid;
			isValid = isValidSurroundChar(s, '(', ')') && isValid;

			// インライン・タグの対応を検査
			isValid = isValidSurroundTag(s, "a") && isValid; //$NON-NLS-1$
			isValid = isValidSurroundTag(s, "i") && isValid; //$NON-NLS-1$
			isValid = isValidSurroundTag(s, "font") && isValid; //$NON-NLS-1$

			// 正しい場合はリストに追加、不正な場合は次の要素と連結
			if (isValid) {
				add(s);
				prev = ""; //$NON-NLS-1$
			} else {
				prev = s;
			}
		}
		if (!prev.equals("")) { //$NON-NLS-1$
			add(prev);
		}
	}

	/**
	 * 括弧などの囲み文字の対が正しいか判定します。
	 * @param text テキスト
	 * @param c1 囲み文字の開始文字
	 * @param c2 囲み文字の終了文字
	 * @return 正しい場合は true
	 */
	protected boolean isValidSurroundChar(final String text, final char c1, final char c2) {
		int c1Count = 0;
		int c2Count = 0;
		for (final char c : text.toCharArray()) {
			if (c == c1) {
				c1Count++;
			} else if (c == c2) {
				c2Count++;
			}
		}
		return c1Count == c2Count;
	}

	/**
	 * 開始終了タグが対で含まれているか判定します。
	 * @param text テキスト
	 * @param tag タグ
	 * @return 対になっている場合は true
	 */
	protected boolean isValidSurroundTag(final String text, final String tag) {

		// 開始タグの数を正規表現でカウント
		// "<" の後ろに / がなく、それに続く ">" 以外の文字列で ">" までを開始タグと判断
		// 例）<span>
		// 例）<a href="...">
		final int startTagCount = countMatchs(text, "<" + tag + "(|\\s+[^>]*)>"); //$NON-NLS-1$ //$NON-NLS-2$

		// 終了タグの数を正規表現でカウント
		// "<" の次に "/" があり、タグ名の次の文字が ">" であるものが終了タグと判断
		// 例）</span>
		// 例）</a>
		final int endTagCount = countMatchs(text, "</" + tag + ">"); //$NON-NLS-1$ //$NON-NLS-2$
		return startTagCount == endTagCount;
	}

	/**
	 * パターンが出現する回数をカウントします。
	 * @param text テキスト
	 * @param pattern 出現パターン
	 * @return 出現回数
	 */
	private int countMatchs(final String text, final String pattern) {
		final Matcher startTag = Pattern.compile("(?i)(?s)" + pattern).matcher(text); //$NON-NLS-1$
		int count = 0;
		while (startTag.find()) {
			count++;
		}
		return count;
	}
}
